@@ -179,8 +179,10 @@ remove_object (GObject * source, GObject * gobj, gpointer data)
       g_signal_emit (cache, cache_signals [OBJECT_REMOVED], 0, gobj);
       g_hash_table_remove (cache->objects, gobj);
     }
-  else
-    g_queue_remove (cache->add_traversal, gobj);
+  else if (g_queue_remove (cache->add_traversal, gobj))
+    {
+      g_object_unref (gobj);
+    }
 }
 
 static void
@@ -277,12 +279,14 @@ add_pending_items (gpointer data)
   while (!g_queue_is_empty (cache->add_traversal))
     {
       AtkStateSet *set;
-      
+
+      /* cache->add_traversal holds a ref to current */
       current = g_queue_pop_head (cache->add_traversal);
       set = atk_object_ref_state_set (current);
 
       if (set && !atk_state_set_contains_state (set, ATK_STATE_TRANSIENT))
         {
+          /* transfer the ref into to_add */
 	  g_queue_push_tail (to_add, current);
           if (!spi_cache_in (cache, G_OBJECT (current)) &&
               !atk_state_set_contains_state  (set, ATK_STATE_MANAGES_DESCENDANTS) &&
@@ -291,6 +295,11 @@ add_pending_items (gpointer data)
               append_children (current, cache->add_traversal);
             }
         }
+      else
+        {
+          /* drop the ref for the removed object */
+          g_object_unref (current);
+        }
 
       if (set)
         g_object_unref (set);
@@ -422,6 +431,9 @@ spi_cache_foreach (SpiCache * cache, GHFunc func, gpointer data)
 gboolean
 spi_cache_in (SpiCache * cache, GObject * object)
 {
+  if (!cache)
+    return FALSE;
+
   if (g_hash_table_lookup_extended (cache->objects,
                                     object,
                                     NULL,
